/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "MetricProducer.h"

#include "dimension.h"

namespace android {
namespace os {
namespace statsd {

using std::map;

void MetricProducer::onMatchedLogEventLocked(const size_t matcherIndex, const LogEvent& event) {
    uint64_t eventTimeNs = event.GetTimestampNs();
    // this is old event, maybe statsd restarted?
    if (eventTimeNs < mStartTimeNs) {
        return;
    }

    bool condition;
    ConditionKey conditionKey;

    std::unordered_set<HashableDimensionKey> dimensionKeysInCondition;
    if (mConditionSliced) {
        for (const auto& link : mConditionLinks) {
            getDimensionKeysForCondition(event, link, &conditionKey[link.condition()]);
        }
        auto conditionState =
            mWizard->query(mConditionTrackerIndex, conditionKey, mDimensionsInCondition,
                           &dimensionKeysInCondition);
        condition = (conditionState == ConditionState::kTrue);
    } else {
        condition = mCondition;
    }

    vector<DimensionsValue> dimensionInWhatValues;
    if (mDimensionsInWhat.has_field() && mDimensionsInWhat.child_size() > 0) {
        getDimensionKeys(event, mDimensionsInWhat, &dimensionInWhatValues);
    }

    if (dimensionInWhatValues.empty() && dimensionKeysInCondition.empty()) {
        onMatchedLogEventInternalLocked(
            matcherIndex, DEFAULT_METRIC_DIMENSION_KEY, conditionKey, condition, event);
    } else if (dimensionKeysInCondition.empty()) {
        for (const DimensionsValue& whatValue : dimensionInWhatValues) {
            onMatchedLogEventInternalLocked(
                matcherIndex,
                MetricDimensionKey(HashableDimensionKey(whatValue), DEFAULT_DIMENSION_KEY),
                conditionKey, condition, event);
        }
    } else if (dimensionInWhatValues.empty()) {
        for (const auto& conditionDimensionKey : dimensionKeysInCondition) {
            onMatchedLogEventInternalLocked(
                matcherIndex,
                MetricDimensionKey(DEFAULT_DIMENSION_KEY, conditionDimensionKey),
                conditionKey, condition, event);
        }
    } else {
        for (const DimensionsValue& whatValue : dimensionInWhatValues) {
            for (const auto& conditionDimensionKey : dimensionKeysInCondition) {
                onMatchedLogEventInternalLocked(
                    matcherIndex,
                    MetricDimensionKey(HashableDimensionKey(whatValue), conditionDimensionKey),
                    conditionKey, condition, event);
            }
        }
    }
}

}  // namespace statsd
}  // namespace os
}  // namespace android
